<?php

/*
 * Some basic forms.
 *
 * @version	0.9.6
 * @author	Al Kingsley <al@bananahead.com>
 * @license	GPL version 2 or later.
 * @copyright	(c) 2003 Al Kingsley
 * @package	MIS
 * @access	public
 */

require('fpdf.php');
define('EURO', chr(128));

class BhForm extends FPDF
{
   // private variables
   var $columns;
   var $format;
   var $angle=0;

   // private functions
   function RoundedRect($x, $y, $w, $h, $r, $style = '')
   {
	   $k = $this->k;
   	$hp = $this->h;
	   if($style=='F')
		   $op='f';
   	elseif($style=='FD' or $style=='DF')
	   	$op='B';
	   else
		   $op='S';
	   $MyArc = 4/3 * (sqrt(2) - 1);
   	$this->_out(sprintf('%.2f %.2f m',($x+$r)*$k,($hp-$y)*$k ));
	   $xc = $x+$w-$r ;
   	$yc = $y+$r;
	   $this->_out(sprintf('%.2f %.2f l', $xc*$k,($hp-$y)*$k ));

   	$this->_Arc($xc + $r*$MyArc, $yc - $r, $xc + $r, $yc - $r*$MyArc, $xc + $r, $yc);
	   $xc = $x+$w-$r ;
   	$yc = $y+$h-$r;
	   $this->_out(sprintf('%.2f %.2f l',($x+$w)*$k,($hp-$yc)*$k));
   	$this->_Arc($xc + $r, $yc + $r*$MyArc, $xc + $r*$MyArc, $yc + $r, $xc, $yc + $r);
	   $xc = $x+$r ;
   	$yc = $y+$h-$r;
	   $this->_out(sprintf('%.2f %.2f l',$xc*$k,($hp-($y+$h))*$k));
   	$this->_Arc($xc - $r*$MyArc, $yc + $r, $xc - $r, $yc + $r*$MyArc, $xc - $r, $yc);
	   $xc = $x+$r ;
   	$yc = $y+$r;
	   $this->_out(sprintf('%.2f %.2f l',($x)*$k,($hp-$yc)*$k ));
   	$this->_Arc($xc - $r, $yc - $r*$MyArc, $xc - $r*$MyArc, $yc - $r, $xc, $yc - $r);
	   $this->_out($op);
   }

   function _Arc($x1, $y1, $x2, $y2, $x3, $y3)
   {
	   $h = $this->h;
	   $this->_out(sprintf('%.2f %.2f %.2f %.2f %.2f %.2f c ', $x1*$this->k, ($h-$y1)*$this->k,
						$x2*$this->k, ($h-$y2)*$this->k, $x3*$this->k, ($h-$y3)*$this->k));
   }

   function Rotate($angle,$x=-1,$y=-1)
   {
	   if($x==-1)
		   $x=$this->x;
   	if($y==-1)
	   	$y=$this->y;
	   if($this->angle!=0)
		   $this->_out('Q');
   	$this->angle=$angle;
	   if($angle!=0){
		   $angle*=M_PI/180;
   		$c=cos($angle);
	   	$s=sin($angle);
		   $cx=$x*$this->k;
   		$cy=($this->h-$y)*$this->k;
	   	$this->_out(sprintf('q %.5f %.5f %.5f %.5f %.2f %.2f cm 1 0 0 1 %.2f %.2f cm',$c,$s,-$s,$c,$cx,$cy,-$cx,-$cy));
	   }
   }

   function _endpage()
   {
	   if($this->angle!=0){
		   $this->angle=0;
   		$this->_out('Q');
	   }
	   parent::_endpage();
   }

   // public functions
   function sizeOfText($text, $max)
   {
	   $index    = 0;
   	$nb_lines = 0;
	   $loop     = TRUE;
   	while($loop){
		   $pos = strpos($text, "\n");
		   if(!$pos){
   			$loop  = FALSE;
	   		$line = $text;
		   }
		   else{
   			$line=substr($text, $index, $pos);
	   		$text=substr($text, $pos+1 );
		   }
		   $length = floor( $this->GetStringWidth($line));
   		$res = 1 + floor($length / $max);
	   	$nb_lines += $res;
	   }
	   return $nb_lines;
   }
   
   function Title($text, $justification, $size)
   {
      
   }
   
   function SubTitle($text, $justification, $size)
   {
      
   }

   // Company
   function addCompany($name, $address, $logo="")
   {
	   $x1 = 10;
   	$y1 = 8;
	   
      $this->SetXY($x1, $y1);
      if(isset($logo)){
         $this->Image($logo, $x1, $y1, 0, 0);
         $y1=$this->GetY()+10;
      }
      $this->SetXY($x1, $y1);
	   $this->SetFont('Arial','B',12);
   	$length = $this->GetStringWidth($name);
	   $this->Cell($length, 2, $name);
   	$this->SetXY($x1, $y1 + 4);
	   $this->SetFont('Arial','',10);
   	$length = $this->GetStringWidth($address);
	   
   	$lines = $this->sizeOfText($address, $length) ;
	   $this->MultiCell($length, 4, $address);
   }
   
   function addDeliveryAddress($name, $address)
   {
	   $x1 = 10;
   	$y1 = 40;
            
   	$this->SetXY($x1, $y1);
	   $this->SetFont('Arial','B',12);
   	$length = $this->GetStringWidth($name);
	   $this->Cell($length, 2, $name);
   	$this->SetXY($x1, $y1 + 4);
	   $this->SetFont('Arial','',10);
   	$length = $this->GetStringWidth($address);
	   
   	$lines = $this->sizeOfText($address, $length) ;
	   $this->MultiCell($length, 4, $address);
   }

   // Label and number of invoice/estimate/po/blah
   function FormNumber($label, $num)
   {
      $r1  = $this->w - 80;
      $r2  = $r1 + 68;
      $y1  = 6;
      $y2  = $y1 + 2;
      $mid = ($r1 + $r2 ) / 2;
    
      $text  = $label . " in " . EURO . " N : " . $num;    
      $szfont = 12;
      $loop   = 0;
    
      while($loop==0){
         $this->SetFont("Helvetica", "B", $szfont);
         $sz = $this->GetStringWidth($text);
         if(($r1+$sz) > $r2)
            $szfont--;
         else
            $loop++;
      }

      $this->SetLineWidth(0.1);
      $this->SetFillColor(192);
      $this->RoundedRect($r1, $y1, ($r2 - $r1), $y2, 2.5, 'DF');
      $this->SetXY( $r1+1, $y1+2);
      $this->Cell($r2-$r1 -1,5, $text, 0, 0, "C" );
   }

   // Estimate
   function addEstimate($numest)
   {
	   $string = sprintf("Est%04d",$numdev);
   	$this->FormNumber("Estimate", $string);
   }

   // Invoice
   function addInvoice($numbill)
   {
	   $string = sprintf("No. %04d", $numbill);
   	$this->addEstimate("Invoice", $string);
   }

   function addDate($date)
   {
	   $r1  = $this->w - 61;
   	$r2  = $r1 + 30;
	   $y1  = 17;
   	$y2  = $y1 ;
	   $mid = $y1 + ($y2 / 2);
   	$this->RoundedRect($r1, $y1, ($r2 - $r1), $y2, 3.5, 'D');
	   $this->Line($r1, $mid, $r2, $mid);
   	$this->SetXY($r1 + ($r2-$r1)/2 - 5, $y1+3 ;
	   $this->SetFont("Helvetica", "B", 10);
   	$this->Cell(10,5, "Date", 0, 0, "C");
	   $this->SetXY( $r1 + ($r2-$r1)/2 - 5, $y1+9 ;
   	$this->SetFont("Helvetica", "", 10);
	   $this->Cell(10,5,$date, 0,0, "C");
   }

   function addClient($ref)
   {
	   $r1  = $this->w - 31;
   	$r2  = $r1 + 19;
	   $y1  = 17;
   	$y2  = $y1;
	   $mid = $y1 + ($y2 / 2);
   	$this->RoundedRect($r1, $y1, ($r2 - $r1), $y2, 3.5, 'D');
	   $this->Line($r1, $mid, $r2, $mid);
   	$this->SetXY($r1 + ($r2-$r1)/2 - 5, $y1+3);
	   $this->SetFont( "Helvetica", "B", 10);
   	$this->Cell(10,5, "Client", 0, 0, "C");
	   $this->SetXY( $r1 + ($r2-$r1)/2 - 5, $y1 + 9 );
   	$this->SetFont("Helvetica", "", 10);
	   $this->Cell(10,5,$ref, 0,0, "C");
   }

   function addPageNumber($page, $of)
   {
	   $r1  = $this->w - 80;
   	$r2  = $r1 + 19;
	   $y1  = 17;
   	$y2  = $y1;
	   $mid = $y1 + ($y2 / 2);
   	$this->RoundedRect($r1, $y1, ($r2 - $r1), $y2, 3.5, 'D');
	   $this->Line($r1, $mid, $r2, $mid);
   	$this->SetXY($r1 + ($r2-$r1)/2 - 5, $y1+3);
	   $this->SetFont("Helvetica", "B", 10);
   	$this->Cell(10,5, "Page", 0, 0, "C");
	   $this->SetXY($r1 + ($r2-$r1)/2 - 5, $y1 + 9);
   	$this->SetFont( "Helvetica", "", 10);
	   $this->Cell(10,5,$page." of ".$of, 0,0, "C");
   }

   // Client address
   function addClientAdress($address)
   {
	   $r1     = $this->w - 80;
   	$r2     = $r1 + 68;
	   $y1     = 40;
   	$this->SetXY( $r1, $y1);
	   $this->MultiCell(60, 4, $address);
   }

   function addPaymentType( $mode )
   {
	   $r1  = 10;
   	$r2  = $r1 + 60;
	   $y1  = 80;
   	$y2  = $y1+10;
	   $mid = $y1 + (($y2-$y1) / 2);
   	$this->RoundedRect($r1, $y1, ($r2 - $r1), ($y2-$y1), 2.5, 'D');
	   $this->Line( $r1, $mid, $r2, $mid);
   	$this->SetXY( $r1 + ($r2-$r1)/2 -5 , $y1+1 );
	   $this->SetFont( "Helvetica", "B", 10);
   	$this->Cell(10,4, "Payment Type", 0, 0, "C");
	   $this->SetXY( $r1 + ($r2-$r1)/2 -5 , $y1 + 5 );
   	$this->SetFont( "Helvetica", "", 10);
	   $this->Cell(10,5,$mode, 0,0, "C");
   }

   // Expiry date
   function addExpiry($date)
   {
      $r1  = 80;
      $r2  = $r1 + 40;
      $y1  = 80;
      $y2  = $y1+10;
      $mid = $y1 + (($y2-$y1) / 2);
      $this->RoundedRect($r1, $y1, ($r2 - $r1), ($y2-$y1), 2.5, 'D');
      $this->Line( $r1, $mid, $r2, $mid);
      $this->SetXY( $r1 + ($r2 - $r1)/2 - 5 , $y1+1 );
      $this->SetFont( "Helvetica", "B", 10);
      $this->Cell(10,4, "Expiry Date", 0, 0, "C");
      $this->SetXY( $r1 + ($r2-$r1)/2 - 5 , $y1 + 5 );
      $this->SetFont( "Helvetica", "", 10);
      $this->Cell(10,5,$date, 0,0, "C");
   }
   
   function addTax($ax)
   {
      $this->SetFont( "Helvetica", "B", 10);
      $r1  = $this->w - 80;
      $r2  = $r1 + 70;
      $y1  = 80;
      $y2  = $y1+10;
      $mid = $y1 + (($y2-$y1) / 2);
      $this->RoundedRect($r1, $y1, ($r2 - $r1), ($y2-$y1), 2.5, 'D');
      $this->Line( $r1, $mid, $r2, $mid);
      $this->SetXY( $r1 + 16 , $y1+1 );
      $this->Cell(40, 4, "VAT Number", '', '', "C");
      $this->SetFont( "Helvetica", "", 10);
      $this->SetXY( $r1 + 16 , $y1+5 );
      $this->Cell(40, 5, $tax, '', '', "C");
   }
   
   function addReference($ref)
   {
      $this->SetFont( "Helvetica", "", 10);
      $length = $this->GetStringWidth( "Reference : " . $ref );
      $r1  = 10;
      $r2  = $r1 + $length;
      $y1  = 92;
      $y2  = $y1+5;
      $this->SetXY( $r1 , $y1 );
      $this->Cell($length,4, "Reference : " . $ref);
   }
   
   function addCols($tab)
   {
      global $columns;
      
      $r1  = 10;
      $r2  = $this->w - ($r1 * 2) ;
      $y1  = 100;
      $y2  = $this->h - 50 - $y1;
      $this->SetXY( $r1, $y1 );
      $this->Rect( $r1, $y1, $r2, $y2, "D");
      $this->Line( $r1, $y1+6, $r1+$r2, $y1+6);
      $colX = $r1;
      $columns = $tab;
      while (list($lib, $pos) = each($tab)){
         $this->SetXY( $colX, $y1+2 );
         $this->Cell( $pos, 1, $lib, 0, 0, "C");
         $colX += $pos;
         $this->Line( $colX, $y1, $colX, $y1+$y2);
      }
   }
   
   function addLineFormat($tab)
   {
      global $format, $columns;
      
      while(list($lib, $pos) = each($columns)){
         if(isset($tab["$lib"]))
            $format[$lib] = $tab["$lib"];
      }
   }
   
   function lineHorz($tab)
   {
      global $columns;
   
      reset( $columns );
      $maxSize=0;
      while ( list( $lib, $pos ) = each($columns)){
         $texte = $tab[ $lib ];
         $longCell  = $pos -2;
         $size = $this->sizeOfText( $texte, $longCell );
         if ($size > $maxSize)
            $maxSize = $size;
      }
      return $maxSize;
   }
   
   function addLine($line, $tab)
   {
      global $columns, $format;
   
      $ordonnee     = 10;
      $maxSize      = $lign;
   
      reset($columns);
      while(list($lib, $pos) = each($columns)){
         $longCell  = $pos -2;
         $text      = $tab[$lib];
         $length    = $this->GetStringWidth($text);
         $sizeTexte = $this->sizeOfText($text, $length);
         $formText  = $format[$lib];
         $this->SetXY($ordonnee, $line-1);
         $this->MultiCell($longCell, 4 , $text, 0, $formText);
         if($maxSize < ($this->GetY()))
            $maxSize = $this->GetY();
         $ordonnee += $pos;
      }
      return ($maxSize - $line);
   }
   
   function SetLineDash($black=false,$white=false)
	{
		if($black and $white)
			$s=sprintf('[%.3f %.3f] 0 d',$black*$this->k,$white*$this->k);
		else
			$s='[] 0 d';
		$this->_out($s);
	}
   
   function addRemark($remark)
   {
      $this->SetFont( "Helvetica", "", 10);
      $length = $this->GetStringWidth( "Remark : " . $remark);
      $r1  = 10;
      $r2  = $r1 + $length;
      $y1  = $this->h - 45.5;
      $y2  = $y1+5;
      $this->SetXY($r1 , $y1);
      $this->Cell($length,4, "Remark : " . $remark);
   }
   
   // call this method first
   function Watermark($text)
   {
      $this->SetFont('Arial','B',50);
      $this->SetTextColor(203,203,203);
      $this->Rotate(45,55,190);
      $this->Text(55,190,$text);
      $this->Rotate(0);
      $this->SetTextColor(0,0,0);
   }
}
?>
